@@ -3,6 +3,17 @@ from inspect import signature
 from typing import Any
 
 from awslambdaric.lambda_context import LambdaContext
+from openapi_schema_pydantic.v3.v3_0_3 import (
+    Info,
+    OpenAPI,
+    Operation,
+    PathItem,
+    Response,
+)
+from openapi_schema_pydantic.v3.v3_0_3.util import (
+    PydanticSchema,
+    construct_open_api_with_schema_class,
+)
 from pydantic import BaseModel, create_model
 
 from pydantic_lambda_handler.main import PydanticLambdaHandler
@@ -14,45 +25,56 @@ class APIGenerationHook(BaseHook):
 
     title: str
     version: str
-    paths: dict[str, Any] = {}
+    paths: dict[str, PathItem] = {}
+    method = None
 
     @classmethod
     def method_init(cls, **kwargs):
         app: PydanticLambdaHandler = kwargs["self"]
         status_code = kwargs["status_code"]
         open_api_status_code = str(int(status_code))
-        method = kwargs["method"]
+        cls.method = kwargs["method"]
+        response_model = kwargs["response_model"]
+
         APIGenerationHook.title = app.title
         APIGenerationHook.version = app.version
 
+        if response_model:
+            schema = {"schema": PydanticSchema(schema_class=response_model)}
+        else:
+            schema = {}
+
         url = kwargs["url"]
         if url in cls.paths:
-            cls.paths[url].update(
-                {
-                    method: {
-                        "responses": {
-                            open_api_status_code: {
-                                "description": kwargs["description"],
-                                "content": {"application/json": {}},
-                            }
-                        },
+            setattr(
+                cls.paths[url],
+                cls.method,
+                Operation(
+                    responses={
+                        open_api_status_code: Response(
+                            description=kwargs["description"],
+                            content={"application/json": schema},
+                        )
                     }
-                }
+                ),
             )
+
         else:
-            cls.paths[url] = {
-                method: {
-                    "responses": {
-                        open_api_status_code: {
-                            "description": kwargs["description"],
-                            "content": {"application/json": {}},
+            cls.paths[url] = PathItem(
+                **{
+                    cls.method: Operation(
+                        responses={
+                            open_api_status_code: Response(
+                                description=kwargs["description"],
+                                content={"application/json": schema},
+                            )
                         }
-                    },
-                }
-            }
+                    )
+                },
+            )
 
         if kwargs["operation_id"]:
-            cls.paths[url][method]["operationId"] = kwargs["operation_id"]
+            getattr(cls.paths[url], cls.method).operationId = kwargs["operation_id"]
 
     @classmethod
     def pre_path(cls, **kwargs) -> None:
@@ -61,7 +83,6 @@ class APIGenerationHook(BaseHook):
         if sig.parameters:
 
             url = kwargs["url"]
-            method = kwargs["method"]
 
             path_model_dict = {}
             query_model_dict = {}
@@ -75,6 +96,9 @@ class APIGenerationHook(BaseHook):
                 raise ValueError(f"re-declared path variable: {url}")
 
             for param, param_info in sig.parameters.items():
+                if issubclass(param_info.annotation, LambdaContext):
+                    continue
+
                 if param in path_parameters:
                     if param_info.annotation == param_info.empty:
                         annotations = str, ...
@@ -96,7 +120,7 @@ class APIGenerationHook(BaseHook):
                     if issubclass(model, BaseModel):
                         if body_model:
                             raise ValueError("Can only use one Pydantic model for body only")
-                        body_model = model  # type: ignore
+                        body_model: BaseModel = model  # type: ignore
                         body_model._alias = param  # type: ignore
                     else:
                         query_model_dict[param] = annotations
@@ -117,10 +141,10 @@ class APIGenerationHook(BaseHook):
 
                 properties.append(p)
 
-            cls.paths[url][method]["parameters"] = properties
+            getattr(cls.paths[url], cls.method).parameters = properties  # type: ignore
             if body_model:
-                cls.paths[url][method]["requestBody"] = {
-                    "content": {"application/json": {"schema": body_model.schema()}}  # type: ignore
+                getattr(cls.paths[url], cls.method).requestBody = {  # type: ignore
+                    "content": {"application/json": {"schema": PydanticSchema(schema_class=body_model)}}
                 }
 
     @classmethod
@@ -133,8 +157,9 @@ class APIGenerationHook(BaseHook):
 
     @classmethod
     def generate(cls):
-        return {
-            "openapi": "3.0.2",
-            "info": {"title": cls.title, "version": cls.version},
-            "paths": cls.paths,
-        }
+        open_api = OpenAPI(
+            info=Info(title=cls.title, version=cls.version),
+            paths=cls.paths,
+        )
+        open_api = construct_open_api_with_schema_class(open_api)
+        return open_api.json(by_alias=True, exclude_none=True, indent=2)
